<?php
namespace BotGuardLoadBalancer;
/*
 * This file is an implementation of the BotGuard Load Balancer API
 *
 * Copyright (c) 2024 Dennis Prochko <wolfsoft@mail.ru>
*/

class BotGuard {

	const API_URL = getenv('BOTGUARD_API_URL') ?? 'https://apiv2.botguard.net';
	const LB_API_URL = getenv('BOTGUARD_LB_API_URL') ?? 'https://localhost';

	private $api_key;

	/**
	 * Constructor. Initialize the instance of the BotGuard class.
	 *
	 * @param string $api_key BotGuard API Key to access all functions
	 */
	public function __construct(string $api_key) {
		$this->api_key = $api_key;
	}

	/**
	 * Get a list of all registered domains in your BotGuard account.
	 *
	 * @return An array of JSON objects
	 */
	public function getDomains() {
		$result = $this->doRequest(
			'GET',
			"website"
		);
		if (!$result) {
			throw new \Exception('There are no domains registered.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}

		return $result;
	}

	/**
	 * Add domain into your BotGuard account.
	 *
	 * @param string $domain Domain name
	 * @param array $options core rules mitigation options
	 */
	public function addDomain(string $domain, array $options = []) {
		$result = $this->doRequest(
			'POST',
			"website", [
				'url' => trim($domain)
			] + $options
		);
		if (!$result) {
			throw new \Exception('Domain registration error. Please go back and try again.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}
	}

	/**
	 * Get an information about domain in your BotGuard account.
	 *
	 * @param string $domain Domain name
	 *
	 * @return JSON object
	 */
	public function getDomain(string $domain) {
		$d = urlencode(trim($domain));
		$result = $this->doRequest(
			'GET',
			"website/$d"
		);
		if (!$result) {
			throw new \Exception('Domain is not registered.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}

		return $result;
	}

	/**
	 * Delete domain from your BotGuard account.
	 *
	 * @param string $domain Domain name
	 */
	public function deleteDomain(string $domain) {
		$d = urlencode(trim($domain));
		$result = $this->doRequest(
			'DELETE',
			"website/$d"
		);
		if (!$result) {
			throw new \Exception('Domain deregistration error. Please go back and try again.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}
	}

	/**
	 * Suspend domain in your BotGuard account.
	 *
	 * @param string $domain Domain name
	 */
	public function suspendDomain(string $domain) {
		$d = urlencode(trim($domain));
		$result = $this->doRequest(
			'PUT',
			"website/$d", [
				'status' => 'paused'
			]
		);
		if (!$result) {
			throw new \Exception('Domain suspending error. Please go back and try again.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}
	}

	/**
	 * Unsuspend (bring online) domain in your BotGuard account.
	 *
	 * @param string $domain Domain name
	 */
	public function unsuspendDomain(string $domain) {
		$d = urlencode(trim($domain));
		$result = $this->doRequest(
			'PUT',
			"website/$d", [
				'status' => 'online'
			]
		);
		if (!$result) {
			throw new \Exception('Domain unsuspending error. Please go back and try again.');
		}
		if (property_exists($result, 'status') && $result->status == 'error') {
			throw new \Exception($result->message);
		}
	}

	/**
	 * Get a link to the Rules Editor page for iframe embedding.
	 *
	 * @param string $domain Domain name
	 *
	 * @return An URL to the page
	 */
	public function getRulesEditorPageLink(string $domain, string $lang = 'en') {
		$d = urlencode(trim($domain));
		return "{${self::API_URL}}/$lang/website/$d/edit?api-key=$this->api_key";
	}

	/**
	 * Get a link to the Events page for iframe embedding.
	 *
	 * @param string $domain Domain name
	 *
	 * @return An URL to the page
	 */
	public function getStatisticsPageLink(string $domain, string $lang = 'en') {
		$d = urlencode(trim($domain));
		return "{${self::API_URL}}/$lang/website/$d/statistics?api-key=$this->api_key";
	}

	/**
	 * Get a link to the Events page for iframe embedding.
	 *
	 * @param string $domain Domain name
	 *
	 * @return An URL to the page
	 */
	public function getEventsPageLink(string $domain, string $lang = 'en') {
		$d = urlencode(trim($domain));
		return "{${self::API_URL}}/$lang/website/$d/events?api-key=$this->api_key";
	}

	/**
	 * Make a generic https request to the BotGuard API endpoint.
	 *
	 * @param string $method HTTP method (GET, POST, ...)
	 * @param string $url URL to make request to
	 * @param array $params An array of "name=value" parameters to pass into the request (optional)
	 *
	 * @return JSON object
	 */
	private function doRequest(string $method, string $url, array $params = null) {
		$url = "{${self::LB_API_URL}}/$url";
		$http = [
			'protocol_version' => '2.0',
			'header' => "Accept: application/json\r\nAuthorization: Bearer {$this->api_key}\r\n",
			'method' => $method,
			'ignore_errors' => true
		];
		if ($params !== null) {
			$http['header'] .= "Content-type: application/x-www-form-urlencoded\r\n";
			$http['content'] = http_build_query($params);
		}
		$context = stream_context_create([
			'http' => $http,
			'ssl' => [
				'peer_name' => parse_url(self::LB_API_URL, PHP_URL_HOST),
				'verify_depth' => 3,
				'SNI_enabled' => true
			]
		]);
		return @json_decode(@file_get_contents($url, false, $context));
	}

}
